/*
    Project: TCTEMP
    Version: V1.5
    File   : tc_gfmul.c

    Copyright (c) 2006-2007 Author of TCTEMP. All rights reserved.
    Public Key Fingerprint: 75EB 6BC2 01B7 F6E7 4BD7 CC58 4A5F C393 19EE 6E69

    Covered by TrueCrypt License 2.2 the full text of which is contained
    in the file License.txt included in TCTEMP binary and source code
    distribution archives.

    The copyright notice of this source code file and the public key
    fingerprint must be retained in full.
*/

#include "tc_priv.h"
#include <string.h>

void GfMul128(unsigned __int8 *T, const unsigned __int8 *Key2,
  const unsigned __int8 *I)
{
    unsigned __int8       P;
    const unsigned __int8 *I_end;
    unsigned __int8       i_bitmask;
    unsigned              k;

    memset(T, '\0', 16);
    for (I_end = I + 8; I != I_end; I++) {
        if (*I) {
            break;
        }
    }
    if (I == I_end) {
        return;
    }
    for (i_bitmask = 0x80; ; i_bitmask >>= 1) {
        if (*I & i_bitmask) {
            goto add_polynomial;
        }
    }

    for(;;) {
        if (*I & i_bitmask) {
add_polynomial:
            Xor128(T, Key2);
        }
        i_bitmask >>= 1;
        if (i_bitmask == 0) {
            if (++I == I_end) break;
            i_bitmask = 0x80;
        }
        P = (T[0] & 0x80) ? 0x87 : 0x00;
        for (k = 0; k < 15; k++) {
            T[k] <<= 1;
            if (T[k+1] & 0x80) T[k] |= 0x01;
        }
        T[15] <<= 1;
        T[15] ^= P;
    }
}

void GfMul64(unsigned __int8 *T, const unsigned __int8 *Key2,
  const unsigned __int8 *I)
{
    unsigned __int8       P;
    const unsigned __int8 *I_end;
    unsigned __int8       i_bitmask;
    unsigned              k;

    memset(T, '\0', 8);

    for (I_end = I + 8; I != I_end; I++) {
        if (*I) break;
    }
    if (I == I_end) {
        return;
    }
    for (i_bitmask = 0x80; ; i_bitmask >>= 1) {
        if (*I & i_bitmask) {
            goto add_polynomial;
        }
    }

    for (;;) {
        if (*I & i_bitmask) {
add_polynomial:
            Xor64(T, Key2);
        }
        i_bitmask >>= 1;
        if (i_bitmask == 0) {
            if (++I == I_end) break;
            i_bitmask = 0x80;
        }
        P = (T[0] & 0x80) ? 0x1b : 0x00;
        for (k = 0; k < 7; k++) {
            T[k] <<= 1;
            if (T[k+1] & 0x80) T[k] |= 0x01;
        }
        T[7] <<= 1;
        T[7] ^= P;
    }
}
