config_name = {
	rel     = 'Release',
	chk     = 'Checked Release',
	dbg     = 'Debug',
	x64_rel = 'x64-PSDK Release',
	x64_chk = 'x64-PSDK Checked Release',
	x64_dbg = 'x64-PSDK Debug'
}

--------------------------------------------------------------------------------

function table_contains(tab, str)
	for i,v in ipairs(tab) do
		if v == str then
			return true
		end
	end
	return false
end

function table_append(dest, src)
	for i,v in ipairs(src) do
		table.insert(dest, v)
	end
end

--------------------------------------------------------------------------------

if not debug_defines then
	debug_defines = { }
end
table.insert(debug_defines, 'DEBUG')

if not ndebug_defines then
	ndebug_defines = { }
end
table.insert(ndebug_defines, 'NDEBUG')

if name then
	project.name = name
end
if not objdir then
	objdir = 'obj/$(ProjectName)/$(ConfigurationName)/$(PlatformName)'
end

--------------------------------------------------------------------------------

i = 1
configs = {}
for k,v in pairs(config_name) do
	configs[i] = v
	i = i + 1
end
project.configs = configs
for k,v in pairs(project.configs) do
	project.config[v].bindir = objdir
end

console          = table_contains(buildflags, 'console')
dll              = table_contains(buildflags, 'dll')
lib              = table_contains(buildflags, 'lib')
static_runtime   = table_contains(buildflags, 'static-runtime')
unicode          = table_contains(buildflags, 'unicode')
no_frame_pointer = table_contains(buildflags, 'no-frame-pointer')
no_import_lib    = table_contains(buildflags, 'no-import-lib')
optimize         = table_contains(buildflags, 'optimize')
optimize_size    = table_contains(buildflags, 'optimize-size')
optimize_speed   = table_contains(buildflags, 'optimize-speed')

if optimize_size and not optimize_speed then
	optimize = 'optimize-size'
elseif not optimize_size and optimize_speed then
	optimize = 'optimize-speed'
elseif (optimize_size and optimize_speed) or optimize then
	optimize = 'optimize'
end

kind = 'winexe'
if dll then
	kind = 'dll'
	for k,v in pairs(project.configs) do
		project.config[v].libdir = objdir
	end
elseif lib then
	kind = 'lib'
elseif console then
	kind = 'exe'
end

--------------------------------------------------------------------------------

package = newpackage();
package.name = project.name
package.kind = kind
package.language = 'c'

if includepaths then
	package.includepaths = includepaths
end
if files then
	package.files = files
end

for k,v in pairs(project.configs) do
	package.config[v].objdir = objdir
	package.config[v].defines = { }
	package.config[v].buildflags = { }
	package.config[v].buildoptions = { }
	package.config[v].linkoptions = { }
end

if unicode then
	table.insert(package.defines, 'UNICODE')
	table.insert(package.defines, '_UNICODE')
end
if console then
	table.insert(package.defines, '_CONSOLE')
end
if defines then
	for i,v in ipairs(defines) do
		table.insert(package.defines, v)
	end
end

package.buildflags = { 
	'no-main',
	'no-rtti',
	'no-exceptions',
	'no-manifest'
}
if static_runtime then
	table.insert(package.buildflags, 'static-runtime')
end
if no_import_lib then
	table.insert(package.buildflags, 'no-import-lib')
end

if links then
	package.links = links
end

package.linkoptions = { 
	'/OPT:NOWIN98' 
}

--------------------------------------------------------------------------------

cfg = config_name['rel']
if cfg then
	table_append(package.config[cfg].defines, ndebug_defines)
	table.insert(package.config[cfg].defines, 'WIN32')
	table.insert(package.config[cfg].buildflags, 'no-symbols')
	if optimize then
		table.insert(package.config[cfg].buildflags, optimize)
	end
	if no_frame_pointer then
		table.insert(package.config[cfg].buildflags, 'no-frame-pointer')
	end
end

cfg = config_name['dbg']
if cfg then
	table_append(package.config[cfg].defines, debug_defines)
	table.insert(package.config[cfg].defines, 'WIN32')
end

cfg = config_name['chk']
if cfg then
	table_append(package.config[cfg].defines, debug_defines)
	table.insert(package.config[cfg].defines, 'WIN32')
	table.insert(package.config[cfg].buildflags, 'no-symbols')
end
--------------------------------------------------------------------------------

cfg = config_name['x64_rel']
if cfg then
	table_append(package.config[cfg].defines, ndebug_defines)
	table.insert(package.config[cfg].defines, 'WIN64')
	table.insert(package.config[cfg].buildflags, 'no-symbols')
	if optimize then
		table.insert(package.config[cfg].buildflags, optimize)
	end
	if no_frame_pointer then
		table.insert(package.config[cfg].buildflags, 'no-frame-pointer')
	end
	table.insert(package.config[cfg].linkoptions, '/MACHINE:x64')
end

cfg = config_name['x64_dbg']
if cfg then
	table_append(package.config[cfg].defines, debug_defines)
	table.insert(package.config[cfg].defines, 'WIN64')
	table.insert(package.config[cfg].linkoptions, '/MACHINE:x64')
end

cfg = config_name['x64_chk']
if cfg then
	table_append(package.config[cfg].defines, debug_defines)
	table.insert(package.config[cfg].defines, 'WIN64')
	table.insert(package.config[cfg].buildflags, 'no-symbols')
	table.insert(package.config[cfg].linkoptions, '/MACHINE:x64')
end
